unit lua_script_provider;

interface

uses
  Windows,
  SysUtils,
  TypInfo,
  CrystalLUA,
  FWDebugerCore;

type
  TScriptProvider = class
  private
    // ,    
    FDebuger: TFWDebugerCore;
    FProcessPath: string;
    FBreakOnEP: Boolean;
    FCurrentThreadIndex: Integer;
    FStopScript: Boolean;
  protected
    //  
    procedure OnBreakPoint(Sender: TObject; ThreadIndex: Integer;
      ExceptionRecord: Windows.TExceptionRecord; BreakPointIndex: Integer;
      var ReleaseBreakpoint: Boolean);
    procedure OnHardwareBreakPoint(Sender: TObject; ThreadIndex: Integer;
      ExceptionRecord: Windows.TExceptionRecord; BreakPointIndex: THWBPIndex;
      var ReleaseBreakpoint: Boolean);
    procedure OnUnknownBreakPoint(Sender: TObject; ThreadIndex: Integer;
      ExceptionRecord: Windows.TExceptionRecord);
    procedure OnIDLE(Sender: TObject);
  public
    //  ,    
    procedure ExecuteScript(Debuger: TFWDebugerCore; const ExePath, ScriptPath: string);
  public
    //     
    procedure InitProcess(BreakOnEP: Boolean);
    function GetProcess: THandle;
    function Run: Boolean;
    procedure SetHardwareBreakpoint(Address: Pointer;
      Size: THWBPSize; Mode: THWBPMode; HWIndex: THWBPIndex);
    procedure SetFlag(Flag: DWORD; Value: Boolean);
    procedure StopScript;
  end;

  function ScriptProvider: TScriptProvider;

implementation

uses
  lua_global_proc,
  debuger_utils;

var
  _ScriptProvider: TScriptProvider = nil;

function ScriptProvider: TScriptProvider;
begin
  if _ScriptProvider = nil then
    _ScriptProvider := TScriptProvider.Create;
  Result := _ScriptProvider;
end;

{ TScriptProvider }

procedure TScriptProvider.ExecuteScript(Debuger: TFWDebugerCore;
  const ExePath, ScriptPath: string);
begin
  FDebuger := Debuger;
  FDebuger.OnBreakPoint := OnBreakPoint;
  FDebuger.OnHardwareBreakPoint := OnHardwareBreakPoint;
  FDebuger.OnIdle := OnIdle;
  FDebuger.OnUnknownBreakPoint := OnUnknownBreakPoint;
  FProcessPath := ExePath;
  LuaScript.LoadScript(ScriptPath);
end;

function TScriptProvider.GetProcess: THandle;
begin
  Result := FDebuger.DebugProcessData.CreatedProcessHandle;
end;

procedure TScriptProvider.InitProcess(BreakOnEP: Boolean);
begin
  FBreakOnEP := BreakOnEP;
  if FDebuger.DebugProcessData.ProcessID = 0 then
    if not FDebuger.DebugNewProcess(FProcessPath, True) then
      RaiseLastOSError;
  FDebuger.CloseDebugProcessOnFree := luaCloseDebugProcessOnFree;
end;

procedure TScriptProvider.OnBreakPoint(Sender: TObject; ThreadIndex: Integer;
  ExceptionRecord: Windows.TExceptionRecord; BreakPointIndex: Integer;
  var ReleaseBreakpoint: Boolean);
begin
  FCurrentThreadIndex := ThreadIndex;
  if not FBreakOnEP then
  begin
    ReleaseBreakpoint := True;
    if ExceptionRecord.ExceptionAddress =
      Pointer(FDebuger.DebugProcessData.EntryPoint) then Exit;
  end;
  FDebuger.AbortMainLoop;
end;

procedure TScriptProvider.OnHardwareBreakPoint(Sender: TObject;
  ThreadIndex: Integer; ExceptionRecord: Windows.TExceptionRecord;
  BreakPointIndex: THWBPIndex; var ReleaseBreakpoint: Boolean);
begin
  FCurrentThreadIndex := ThreadIndex;
  FDebuger.AbortMainLoop;
end;

procedure TScriptProvider.OnIDLE(Sender: TObject);
begin
  if FStopScript then
    FDebuger.StopDebug;
end;

procedure TScriptProvider.OnUnknownBreakPoint(Sender: TObject;
  ThreadIndex: Integer; ExceptionRecord: Windows.TExceptionRecord);
var
  Args: TLuaArgs;
begin
  if not LuaScript.ProcExists('OnUnknownBreakPoint') then Exit;
  SetLength(Args, 1);
  Args[0].AsRecord := LuaRecord(@ExceptionRecord, LuaScript.RecordInfo['TExceptionRecord']);
  LuaScript.Call('OnUnknownBreakPoint', Args);
  FDebuger.ContinueStatus := luaContinueStatus;
end;

function TScriptProvider.Run: Boolean;
begin
  FDebuger.RunMainLoop;
  Result := FDebuger.DebugProcessData.AttachedFileHandle <> 0;
end;

procedure TScriptProvider.SetFlag(Flag: DWORD; Value: Boolean);
begin
  FDebuger.SetFlag(FCurrentThreadIndex, Flag, Value);
end;

procedure TScriptProvider.SetHardwareBreakpoint(Address: Pointer;
  Size: THWBPSize; Mode: THWBPMode; HWIndex: THWBPIndex);
begin
  FDebuger.SetHardwareBreakpoint(FCurrentThreadIndex, Address, Size, Mode, HWIndex, '');
end;

procedure TScriptProvider.StopScript;
begin
  FStopScript := True;
  Run;
end;

initialization

finalization

  FreeAndNil(_ScriptProvider);

end.
